import json
import os
import boto3

# from botocore.exceptions import NoCredentialsError


def wasabi_auth(ACCESS_KEY, WASABI_SECRET):
    """
    Creates an authenticated client for interacting with an S3 bucket hosted on the Wasabi cloud storage service.
    
    Args:
    ACCESS_KEY (str): Wasabi Access Key
    WASABI_SECRET (str): Wasabi Secret Key
    
    Returns:
    s3 (boto3.client): An authenticated client object for interacting with Wasabi S3
    
    """

    # Set the Wasabi Access Key and Secret Key
    WASABI_ACCESS_KEY = ACCESS_KEY
    WASABI_SECRET_KEY = WASABI_SECRET

    # Create a session object using the Wasabi Access Key and Secret Key
    session = boto3.Session(
        aws_access_key_id=WASABI_ACCESS_KEY,
        aws_secret_access_key=WASABI_SECRET_KEY,
    )
    
    # Create an S3 client object using the session object and the Wasabi endpoint URL
    s3 = session.client(
        "s3", endpoint_url="https://s3.ap-southeast-1.wasabisys.com"
    )
    
    # Return the S3 client object
    return s3

def wasabi_auth_client():
    pass


# import json

# import boto3
# from logging import logger

from botocore.exceptions import ClientError


def wasabi_download(s3, bucket_name, objects, local_storage_path):
    """
    Function to read and download the files from wasabi cloud storage to local

    Params:
    s3 : s3 client which is invoked from wasabi_auth function
    bucket_name : bucket from which data needs to be downloaded
    objects : list of file paths from bucket_list function
    local_storage_path : path to store the file on local
    """

    pass


def bucket_list(s3, bucket_name, prefix=None):
    """
    Lists the objects in a bucket, optionally filtered by a prefix.

    :param bucket_name: The bucket to query.
    :param prefix: When specified, only objects that start with this prefix are listed.
    :return: The list of objects.
    """
    bucket = s3.Bucket(bucket_name)
    try:
        if not prefix:
            objects = list(bucket.objects.all())
        else:
            objects = list(bucket.objects.filter(Prefix=prefix))
        # logger.info("Got objects %s from bucket '%s'",
        #             [o.key for o in objects], bucket_name.name)
    except ClientError:
        # logger.exception("Couldn't get objects for bucket '%s'.", bucket_name.name)
        raise
    else:
        return objects


def read_data_codeboook(s3, bucket_name, object_list):
    """
    Function to read the csv file and its corresponding codebook
    """
    for each in object_list:
        file_key = s3.get_object(Bucket=bucket_name, key=each)
        print(file_key)
    pass



from setuptools import setup, find_packages

setup(
    name='wasabit',
    version='0.2.1',
    description='This package offers a solution for uploading files, creating folders, and authenticating with Wasabi',
    author='Saurabh Harak',
    author_email='saurabh_harak@isb.edu',
    packages=find_packages(),
    classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
     ],
    install_requires=[
        'boto3==1.26.65',
        'botocore==1.29.65',
        'jmespath==1.0.1',
        'python-dateutil==2.8.1',
        's3transfer==0.6.0',
        'six==1.16.0',
        'urllib3==1.26.14',
        
    ],
)



# import json
import boto3
from boto3.s3.transfer import TransferConfig
from botocore.exceptions import NoCredentialsError, ClientError
from wasabit.wasabi_auth import wasabi_auth
import os
from botocore.exceptions import NoCredentialsError
from pathlib import Path



def upload_to_wasabi(folder_path: str, bucket_name: str, wasabi_path: str, access_key = None, secret_key = None) -> None:
    """
    Uploads all files present in a specific folder to a specified Wasabi bucket and subfolder or prefix.
    :param folder_path: The path of the folder containing the files to be uploaded.
    :param bucket_name: The name of the Wasabi bucket where the files will be uploaded.
    :param wasabi_path: The prefix or subfolder within the bucket where the files will be uploaded.
    :param access_key: The Wasabi access key.
    :param secret_key: The Wasabi secret key.
    """
    transfer_config = TransferConfig(
    multipart_threshold=1024 * 25,  # 25MB
    max_concurrency=10,
    num_download_attempts=10,
)
    try:
        # create an S3 client
        s3 = wasabi_auth(access_key,secret_key)
        for file in os.listdir(folder_path):
            file_path = os.path.join(folder_path, file)
            if os.path.isfile(file_path):
                remote_file = file_path.split('/')[-1]
                remote_file = wasabi_path + remote_file
                # upload the file
                s3.upload_file(file_path, bucket_name,remote_file,Config=transfer_config)
                print(f'{file_path} is uploaded to {remote_file}')
    except NoCredentialsError as e:
        print("Credentials not found, please check your access key and secret key.")
    except ClientError as e:
        if e.response['Error']['Code'] == "InvalidAccessKeyId":
            print("Invalid access key, please check your access key.")
        elif e.response['Error']['Code'] == "SignatureDoesNotMatch":
            print("Invalid secret key, please check your secret key.")
        else:
            print(f'An error occurred: {e}')
    except Exception as e:
        print(f'An error occurred: {e}')

        



def file_upload_to_wasabi(csv_file_path: str, bucket_name: str, wasabi_path: str, access_key, secret_key) -> None:
    """
    Uploads a specific CSV file to a specified Wasabi bucket and subfolder or prefix.
    :param csv_file_path: The full path of the CSV file to be uploaded.
    :param bucket_name: The name of the Wasabi bucket where the file will be uploaded.
    :param wasabi_path: The prefix or subfolder within the bucket where the file will be uploaded.
    :param access_key: The Wasabi access key.
    :param secret_key: The Wasabi secret key.
    """
    transfer_config = TransferConfig(
        multipart_threshold=1024 * 25,  # 25MB
        max_concurrency=10,
        num_download_attempts=10,
    )
    try:
        # create an S3 client
        s3 = wasabi_auth(access_key,secret_key)
        # get the file name from the full path
        file_name = os.path.basename(csv_file_path)
        remote_file = os.path.join(wasabi_path, file_name)
        # upload the file
        s3.upload_file(csv_file_path, bucket_name, remote_file, Config=transfer_config)
        print(f'{csv_file_path} is uploaded to {remote_file}')
    except NoCredentialsError as e:
        print("Credentials not found, please check your access key and secret key.")
    except ClientError as e:
        if e.response['Error']['Code'] == "InvalidAccessKeyId":
            print("Invalid access key, please check your access key.")
        elif e.response['Error']['Code'] == "SignatureDoesNotMatch":
            print("Invalid secret key, please check your secret key.")
        else:
            print(f'An error occurred: {e}')
    except Exception as e:
        print(f'An error occurred: {e}')



import os
import boto3
from botocore.exceptions import NoCredentialsError, ClientError
from pathlib import Path


def upload_folder_to_wasabi(local_folder_path: str, bucket_name: str, wasabi_path: str, access_key: str, secret_key: str) -> None:
    """
    Recursively uploads all files present in a specific folder to a specified Wasabi bucket and subfolder or prefix.
    :param local_folder_path: The path of the local folder containing the files to be uploaded.
    :param bucket_name: The name of the Wasabi bucket where the files will be uploaded.
    :param wasabi_path: The prefix or subfolder within the bucket where the files will be uploaded. Optional.
    :param access_key: The Wasabi access key.
    :param secret_key: The Wasabi secret key.
    """
    transfer_config = TransferConfig(
        multipart_threshold=1024 * 25,  # 25MB
        max_concurrency=10,
        num_download_attempts=10,
    )
    try:
        # create an S3 client
        s3 = boto3.client('s3',
                          endpoint_url='https://s3.ap-southeast-1.wasabisys.com',
                          aws_access_key_id=access_key,
                          aws_secret_access_key=secret_key)

        # convert folder path to platform-independent path
        local_folder_path = Path(local_folder_path).resolve()
        for root, dirs, files in os.walk(local_folder_path):
            for file in files:
                # get local file path and remote file path
                local_file_path = os.path.join(root, file)
                relative_path = os.path.relpath(local_file_path, local_folder_path)
                remote_file_path = os.path.join(wasabi_path, relative_path).replace(os.sep, '/')
                
                # create the target folder if it does not already exist
                if wasabi_path:
                    s3.put_object(Bucket=bucket_name, Key=f"{wasabi_path}/")
                
                # upload the file
                s3.upload_file(local_file_path, bucket_name, remote_file_path, Config=transfer_config)
                print(f'{local_file_path} is uploaded to {remote_file_path}')
    except NoCredentialsError as e:
        print("Credentials not found, please check your access key and secret key.")
    except ClientError as e:
        if e.response['Error']['Code'] == "InvalidAccessKeyId":
            print("Invalid access key, please check your access key.")
        elif e.response['Error']['Code'] == "SignatureDoesNotMatch":
            print("Invalid secret key, please check your secret key.")
        else:
            print(f'An error occurred: {e}')
    except Exception as e:
        print(f'An error occurred: {e}')
